function defuzzed_image = defuzz(filename, extension)
%reduce fuzz on the image using hit-and-miss transforms to clear
%hanging edge pixels and fill in gap edge pixels (thinning/thickening)

%read in the result of the text detection/other potential preprocessing
%should be greyscale with text black and background white

orig = imread([filename extension]);

up_hang_interval = [0 0 0; -1 1 -1; 1 1 1];
down_hang_interval = [1 1 1; -1 1 -1; 0 0 0];
left_hang_interval = [0 -1 1; 0 1 1; 0 -1 1];
right_hang_interval = [1 -1 0; 1 1 0; 1 -1 0];

up_gap_interval = [0 0 0; 1 -1 1; 1 1 1];
down_gap_interval = [1 1 1; 1 -1 1; 0 0 0];
left_gap_interval = [0 1 1; 0 -1 1; 0 1 1];
right_gap_interval = [1 1 0; 1 -1 0; 1 1 0];

orig_inv = (orig == 0);

orig_thickened = (orig_inv + bwhitmiss(orig_inv, up_gap_interval));
orig_thickened = (orig_thickened + bwhitmiss(orig_inv, down_gap_interval));
orig_thickened = (orig_thickened + bwhitmiss(orig_inv, left_gap_interval));
orig_thickened = (orig_thickened + bwhitmiss(orig_inv, right_gap_interval));

orig_thinned = (orig_thickened - bwhitmiss(orig_inv, up_hang_interval));
orig_thinned = (orig_thinned - bwhitmiss(orig_inv, down_hang_interval));
orig_thinned = (orig_thinned - bwhitmiss(orig_inv, left_hang_interval));
orig_thinned = (orig_thinned - bwhitmiss(orig_inv, right_hang_interval));

final = (orig_thinned == 0);

imwrite(final, [filename '_nofuzz.tif'], 'tif', 'Compression', 'none');
