function [new_cc_coordinates new_cc_data new_region_data] = pull_in_stranded_ccs(cc_coordinates, cc_data, region_data, stranded_cc_coordinates, stranded_cc_data, stranded_region_data)
%pull in connected components that were originally discarded (stranded)
%but should still be considered as valid components because of their
%proximity and similarity (size-wise) to other valid connected components

%initialize new arrays with all the data we already know we want to keep
new_cc_coordinates = cc_coordinates;
new_cc_data = cc_data;
new_region_data = region_data;

%mark all the original cc_coordinates as dependent on their region -
%all reclaimed cc_coordinates will be dependent on the region that
%pulled them in
for index = 1:size(new_cc_coordinates, 1)
new_cc_coordinates(index, 8) = new_cc_coordinates(index, 5);
end

%initalize locations of next valid rows in each array
next_cc_coordinates_index = size(new_cc_coordinates, 1) + 1;
next_cc_data_index = size(new_cc_data, 1) + 1;
next_region_data_index = size(new_region_data, 1) + 1;

%treating cc_coordinates as a queue, growing toward higher indices,
%process each set of coordinates - look for stranded coordinates
%that are close enough to each component to be added

processing_curr_index = 1;
while processing_curr_index <= size(new_cc_coordinates, 1)
curr_region = new_cc_data(processing_curr_index, 1);
curr_min_row = new_cc_coordinates(processing_curr_index, 1);
curr_min_col = new_cc_coordinates(processing_curr_index, 2);
curr_max_row = new_cc_coordinates(processing_curr_index, 3);
curr_max_col = new_cc_coordinates(processing_curr_index, 4);
curr_center_row = (curr_min_row + curr_max_row) / 2;
curr_center_col = (curr_min_col + curr_max_col) / 2;
curr_height = curr_max_row - curr_min_row;
curr_width = curr_max_col - curr_min_col;

%loop through all stranded ccs, looking for close ones to add back
processing_stranded_index = 1;
while processing_stranded_index <= size(stranded_cc_coordinates, 1)
stranded_region = stranded_cc_data(processing_stranded_index, 1);
stranded_min_row = stranded_cc_coordinates(processing_stranded_index, 1);
stranded_min_col = stranded_cc_coordinates(processing_stranded_index, 2);
stranded_max_row = stranded_cc_coordinates(processing_stranded_index, 3);
stranded_max_col = stranded_cc_coordinates(processing_stranded_index, 4);
stranded_center_row = (stranded_min_row + stranded_max_row) / 2;
stranded_center_col = (stranded_min_col + stranded_max_col) / 2;
stranded_height = stranded_max_row - stranded_min_row;
stranded_width = stranded_max_col - stranded_min_col;

%determine direction traveled from center of current valid cc to
%center of stranded cc
to_stranded_row = stranded_center_row - curr_center_row;
to_stranded_col = stranded_center_col - curr_center_col;

%treat this as a horizontal neighbor if horizontal distance is greater
if abs(to_stranded_col) > abs(to_stranded_row)

%treat as a left neighbor if horizontal distance is negative
if to_stranded_col < 0

%first criterion - right edge of stranded cc is within 1/2 width
%of current cc from left edge of current cc
if curr_min_col - (curr_width * 0.5) > stranded_max_col
processing_stranded_index = processing_stranded_index + 1;
continue
end

%second criterion - center row of stranded cc is within 1/2 height
%of current cc from center row of current cc
if curr_center_row + (curr_height * 0.5) < stranded_center_row || curr_center_row - (curr_height * 0.5) > stranded_center_row
processing_stranded_index = processing_stranded_index + 1;
continue
end

%third criterion - height of stranded cc is within 1/4 height of current
%cc (makes sure we don't start adding garbage that's not part of the
%current cc's word)
if curr_height * 1.25 < stranded_height || curr_height * 0.75 > stranded_height
processing_stranded_index = processing_stranded_index + 1;
continue
end

%if we've passed all three criteria, add this cc's data to all the proper places

str = 'ding! add it! left'
curr_region
stranded_region
 
new_cc_coordinates(next_cc_coordinates_index, 1:7) = stranded_cc_coordinates(processing_stranded_index, 1:7);
new_cc_coordinates(next_cc_coordinates_index, 8) = curr_region;
next_cc_coordinates_index = next_cc_coordinates_index + 1;
new_cc_data(next_cc_data_index, :) = stranded_cc_data(processing_stranded_index,:);
next_cc_data_index = next_cc_data_index + 1;
found_in_region_data = new_region_data(find(new_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)));
if size(found_in_region_data, 1) == 0 || size(found_in_region_data, 2) == 0
new_region_data(next_region_data_index, :) = stranded_region_data(find(stranded_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)), :);
next_region_data_index = next_region_data_index + 1;
end

%remove this stranded cc from the stranded arrays and don't increment
%the processing_stranded_index
stranded_cc_coordinates(processing_stranded_index, :) = [];
stranded_cc_data(processing_stranded_index, :) = [];

%treat as a right neighbor if horizontal distance is positive
else

%first criterion - left edge of stranded cc is within 1/2 width
%of current cc from right edge of current cc
if curr_max_col + (curr_width * 0.5) < stranded_min_col
processing_stranded_index = processing_stranded_index + 1;
continue
end

%second criterion - center row of stranded cc is within 1/2 height
%of current cc from center row of current cc
if curr_center_row + (curr_height * 0.5) < stranded_center_row || curr_center_row - (curr_height * 0.5) > stranded_center_row
processing_stranded_index = processing_stranded_index + 1;
continue
end

%third criterion - height of stranded cc is within 1/4 height of current
%cc (makes sure we don't start adding garbage that's not part of the
%current cc's word)
if curr_height * 1.25 < stranded_height || curr_height * 0.75 > stranded_height
processing_stranded_index = processing_stranded_index + 1;
continue
end

%if we've passed all three criteria, add this cc's data to all the proper places

str = 'ding! add it! right'
curr_region
stranded_region
 
new_cc_coordinates(next_cc_coordinates_index, 1:7) = stranded_cc_coordinates(processing_stranded_index, 1:7);
new_cc_coordinates(next_cc_coordinates_index, 8) = curr_region;
next_cc_coordinates_index = next_cc_coordinates_index + 1;
new_cc_data(next_cc_data_index, :) = stranded_cc_data(processing_stranded_index,:);
next_cc_data_index = next_cc_data_index + 1;
found_in_region_data = new_region_data(find(new_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)));
if size(found_in_region_data, 1) == 0 || size(found_in_region_data, 2) == 0
new_region_data(next_region_data_index, :) = stranded_region_data(find(stranded_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)), :);
next_region_data_index = next_region_data_index + 1;
end

%remove this stranded cc from the stranded arrays and don't increment
%the processing_stranded_index
stranded_cc_coordinates(processing_stranded_index, :) = [];
stranded_cc_data(processing_stranded_index, :) = [];

%end if to_stranded_col < 0
end

%treat this as a vertical neighbor if vertical distance is greater
else

%treat as an up neighbor if vertical distance is negative
if to_stranded_row < 0

%first criterion - bottom edge of stranded cc is within 1/2 height
%of current cc from top edge of current cc
if curr_min_row - (curr_height * 0.5) > stranded_max_row
processing_stranded_index = processing_stranded_index + 1;
continue
end

%second criterion - center col of stranded cc is within 1/2 width
%of current cc from center col of current cc
if curr_center_col + (curr_width * 0.5) < stranded_center_col || curr_center_col - (curr_width * 0.5) > stranded_center_col
processing_stranded_index = processing_stranded_index + 1;
continue
end

%third criterion - width of stranded cc is within 1/4 width of current
%cc (makes sure we don't start adding garbage that's not part of the
%current cc's word)
if curr_width * 1.25 < stranded_width || curr_width * 0.75 > stranded_width
processing_stranded_index = processing_stranded_index + 1;
continue
end

%if we've passed all three criteria, add this cc's data to all the proper places

str = 'ding! add it! up'
curr_region
stranded_region
 
new_cc_coordinates(next_cc_coordinates_index, 1:7) = stranded_cc_coordinates(processing_stranded_index, 1:7);
new_cc_coordinates(next_cc_coordinates_index, 8) = curr_region;
next_cc_coordinates_index = next_cc_coordinates_index + 1;
new_cc_data(next_cc_data_index, :) = stranded_cc_data(processing_stranded_index,:);
next_cc_data_index = next_cc_data_index + 1;
found_in_region_data = new_region_data(find(new_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)));
if size(found_in_region_data, 1) == 0 || size(found_in_region_data, 2) == 0
new_region_data(next_region_data_index, :) = stranded_region_data(find(stranded_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)), :);
next_region_data_index = next_region_data_index + 1;
end

%remove this stranded cc from the stranded arrays and don't increment
%the processing_stranded_index
stranded_cc_coordinates(processing_stranded_index, :) = [];
stranded_cc_data(processing_stranded_index, :) = [];

%treat as a down neighbor if vertical distance is positive
else

%first criterion - top edge of stranded cc is within 1/2 height
%of current cc from bottom edge of current cc
if curr_max_row + (curr_height * 0.5) < stranded_min_row
processing_stranded_index = processing_stranded_index + 1;
continue
end

%second criterion - center col of stranded cc is within 1/2 width
%of current cc from center col of current cc
if curr_center_col + (curr_width * 0.5) < stranded_center_col || curr_center_col - (curr_width * 0.5) > stranded_center_col
processing_stranded_index = processing_stranded_index + 1;
continue
end

%third criterion - width of stranded cc is within 1/4 width of current
%cc (makes sure we don't start adding garbage that's not part of the
%current cc's word)
if curr_width * 1.25 < stranded_width || curr_width * 0.75 > stranded_width
processing_stranded_index = processing_stranded_index + 1;
continue
end

%if we've passed all three criteria, add this cc's data to all the proper places

str = 'ding! add it! down'
curr_region
stranded_region
 
stranded_cc_coordinates(processing_stranded_index, :)

new_cc_coordinates(next_cc_coordinates_index, 1:7) = stranded_cc_coordinates(processing_stranded_index, 1:7);
new_cc_coordinates(next_cc_coordinates_index, 8) = curr_region;
next_cc_coordinates_index = next_cc_coordinates_index + 1;
new_cc_data(next_cc_data_index, :) = stranded_cc_data(processing_stranded_index,:);
next_cc_data_index = next_cc_data_index + 1;
found_in_region_data = new_region_data(find(new_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)));
if size(found_in_region_data, 1) == 0 || size(found_in_region_data, 2) == 0
new_region_data(next_region_data_index, :) = stranded_region_data(find(stranded_region_data(:,1) == stranded_cc_data(processing_stranded_index, 1)), :);
next_region_data_index = next_region_data_index + 1;
end

%remove this stranded cc from the stranded arrays and don't increment
%the processing_stranded_index
stranded_cc_coordinates(processing_stranded_index, :) = [];
stranded_cc_data(processing_stranded_index, :) = [];

%end if to_stranded_col < 0
end

%end if to stranded_col > to_stranded_row
end

%end while processing_stranded_index <= size(stranded_cc_coordinates, 1)
end

processing_curr_index = processing_curr_index + 1;

%end while processing_curr_index <= size(new_cc_coordinates,1)
end
